/* 
 * mixer_mod.c
 *
 * wave mixing module. mixes active WAVEs in 'waves[]'
 *
 * Copyright (C) 1998 Hermann Samso
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <ncurses.h>
#include "types.h"
#include "delfin.h"
#include "display.h"
#include "audiodrv.h"
#include "wave.h"
#include "module.h"

WAVE* mixed = NULL;
BOOL mixer_inited = FALSE;
BOOL mixer_playing = FALSE;
CHAR fname[43];

void MixerCopyToMixed(WAVE* wave);
void Mixer(WAVE* dest, WAVE* wave);

void MixerInit(void) {
	if (trace_on)
		fprintf(stderr, "MixerInit()\n");


	if (mixer_inited)
		return;

	mixed = WaveNew("mixed", AudioGetBufferSize());
	WaveRegister(mixed);
	mixer_inited = TRUE;
}


void MixerExit(void) {
	if (trace_on)
		fprintf(stderr, "MixerExit()\n");

	WaveFree(mixed);
}


void MixerCtrl(INT input) {
	INT i;
	WAVE* wave = WaveGet();
	INT nr_waves = WaveHowMany();
	INT keys[] = {
		'a',		// toggle WAVE on/off
		's',		// restart WAVE
		'd',		// restart all active WAVEs
		'f', 		// toggle play
		'o',		// dump audio to file
		'O'			// stop dump to file
	};


	if (trace_on)
		fprintf(stderr, "WavCtrl()\n");
		

	if (input == keys[0]) {
		if (wave->active) {
			wave->active = FALSE;
			WaveClear(mixed);
		}
		else
			wave->active = TRUE;
		WaveDisplay();
	}
	else if (input == keys[1]) {
		wave->position = wave->start;
		WaveDisplay();
	}
	else if (input == keys[2]) {
		for (i = 0; i < nr_waves ; i++) {
			wave = WaveGetByIndex(i);
			if (wave->active)
				wave->position = wave->start;
		}
		WaveDisplay();
	}
	else if (input == keys[3]) {
		if (mixer_playing)
			mixer_playing = FALSE;
		else
			mixer_playing = TRUE;
		ModuleDisplay();
	}
	else if (input == keys[4]) {
		echo();
		notimeout(stdscr, TRUE);
		mvprintw(LINES-1, 0, "file:");
		mvgetnstr(LINES-1, 5, fname, 42);
		notimeout(stdscr, FALSE);
		noecho();
		AudioDumpToFile(fname);
		ModuleDisplay();
	}
	else if (input == keys[5]) {
		AudioDumpStop();
		fname[0] = 0;
		ModuleDisplay();
	}
	else
		usleep(100);
}


void MixerExec(void) {
	INT i;
	INT nr_waves = WaveHowMany();
	WAVE* wave;

	if (trace_on)
		fprintf(stderr, "MixerExec(): nr_waves=%d\n", nr_waves);


	// first active WAVE gets copied to 'mixed'
	for (i = 0; i < nr_waves; i++) {
		wave = WaveGetByIndex(i);
		if (wave->active) {
			MixerCopyToMixed(wave);
			break;
		}
	}

	// the following active WAVEs get mixed to 'mixed'
	for (; i < nr_waves; i++) {
		wave = WaveGetByIndex(i);
		if (wave->active)
			Mixer(mixed, wave);
	}


	if (mixer_playing) {
		WavePlay(mixed);
	}
}


/*
 * Mixes 'src' and 'dest' , to size of 'dest'
 *
 */
void Mixer(WAVE* dest, WAVE* src) {
	ULONG i;
//	FLOAT div = (dest->peak + src->peak);
	
	if (trace_on)
		fprintf(stderr, "Mixer()\n");


	if (dest == NULL || dest == NULL) {
		fprintf(stderr, "mixer_mod: Mixer: NULL ptr!\n");
		return;
	}
	
	
	for (i = 0; i < dest->size; i++) {
		dest->samples[i].left =
			(dest->samples[i].left + src->buffer[i].left * src->lvol)/2;
//			(dest->samples[i].left + src->buffer[i].left * src->lvol)/div;
		dest->samples[i].right = 
			(dest->samples[i].right + src->buffer[i].right * src->rvol)/2;
//			(dest->samples[i].right + src->buffer[i].right * src->rvol)/div;
//printf("Mixer: %f\n", mixed->samples[i].left);
	}
}


void MixerCopyToMixed(WAVE* wave) {
	ULONG i;

	if (trace_on)
		fprintf(stderr, "MixerCopyToMixed()\n");

	if (wave == NULL) {
		fprintf(stderr, "wave == NULL\n");
		return;
	}
	else if (mixed == NULL) {
		fprintf(stderr, "mixed == NULL\n");
		return;
	}
	else if (mixed->samples == NULL) {
		fprintf(stderr, "mixed->samples == NULL\n");
		return;
	}
	else if (wave->buffer == NULL) {
		fprintf(stderr, "wave->buffer == NULL\n");
		return;
	}
	
	for (i = 0; i < mixed->size; i++) {
		mixed->samples[i].left = wave->buffer[i].left * wave->lvol;
		mixed->samples[i].right = wave->buffer[i].right * wave->lvol;
//printf("MixCopyTo...: %f\n", wave->buffer[i].left);
	}
}

		
void MixerDisplay(void) {
	if (trace_on)
		fprintf(stderr, "MixerDisplay()\n");

	DisplayClearBottom();
	mvprintw(LINES-2, 0, "[%s]", mixer_mod.name);
	mvprintw(LINES-1, 0, "play:%d", mixer_playing);
	mvprintw(LINES-2, 13, "outfile:%s", fname);
}




MODULE mixer_mod = {
	NULL,			/* next MODULE  in list */
	NULL,			/* previous MODULE in list */
	"mixer",
	"v1998.1.22 by Hermann Samso",
	MixerInit,
	MixerExit,
	MixerCtrl,
	MixerExec,
	MixerDisplay
};
